package sun.java2d.d3d;

import java.awt.Window;
import sun.java2d.ScreenUpdateManager;
import sun.java2d.StateTracker;
import sun.awt.image.DataBufferNative;
import sun.java2d.loops.GraphicsPrimitive;
import sun.awt.image.PixelConverter;
import sun.java2d.pipe.BufferedContext;
import java.awt.GraphicsEnvironment;
import java.awt.GraphicsConfiguration;
import sun.awt.SunToolkit;
import java.awt.Component;
import sun.java2d.pipe.RenderBuffer;
import sun.java2d.loops.MaskFill;
import sun.java2d.loops.CompositeType;
import java.awt.AlphaComposite;
import sun.awt.image.SurfaceManager;
import sun.java2d.SunGraphics2D;
import java.awt.image.SinglePixelPackedSampleModel;
import java.awt.image.DirectColorModel;
import java.awt.image.Raster;
import sun.java2d.InvalidPipeException;
import sun.awt.image.SunVolatileImage;
import java.awt.Rectangle;
import java.awt.BufferCapabilities;
import sun.java2d.SurfaceDataProxy;
import java.awt.image.ColorModel;
import sun.java2d.pipe.ParallelogramPipe;
import sun.java2d.pipe.PixelToParallelogramConverter;
import sun.awt.image.WritableRasterNative;
import sun.java2d.pipe.hw.ExtendedBufferCapabilities;
import java.awt.Image;
import sun.awt.windows.WComponentPeer;
import sun.java2d.loops.SurfaceType;
import sun.java2d.pipe.hw.AccelSurface;
import sun.java2d.SurfaceData;

public class D3DSurfaceData extends SurfaceData implements AccelSurface
{
    public static final int D3D_DEVICE_RESOURCE = 100;
    public static final int ST_INT_ARGB = 0;
    public static final int ST_INT_ARGB_PRE = 1;
    public static final int ST_INT_ARGB_BM = 2;
    public static final int ST_INT_RGB = 3;
    public static final int ST_INT_BGR = 4;
    public static final int ST_USHORT_565_RGB = 5;
    public static final int ST_USHORT_555_RGB = 6;
    public static final int ST_BYTE_INDEXED = 7;
    public static final int ST_BYTE_INDEXED_BM = 8;
    public static final int ST_3BYTE_BGR = 9;
    public static final int SWAP_DISCARD = 1;
    public static final int SWAP_FLIP = 2;
    public static final int SWAP_COPY = 3;
    private static final String DESC_D3D_SURFACE = "D3D Surface";
    private static final String DESC_D3D_SURFACE_RTT = "D3D Surface (render-to-texture)";
    private static final String DESC_D3D_TEXTURE = "D3D Texture";
    static final SurfaceType D3DSurface;
    static final SurfaceType D3DSurfaceRTT;
    static final SurfaceType D3DTexture;
    private int type;
    private int width;
    private int height;
    private int nativeWidth;
    private int nativeHeight;
    protected WComponentPeer peer;
    private Image offscreenImage;
    protected D3DGraphicsDevice graphicsDevice;
    private int swapEffect;
    private ExtendedBufferCapabilities.VSyncType syncType;
    private int backBuffersNum;
    private WritableRasterNative wrn;
    protected static D3DRenderer d3dRenderPipe;
    protected static PixelToParallelogramConverter d3dTxRenderPipe;
    protected static ParallelogramPipe d3dAAPgramPipe;
    protected static D3DTextRenderer d3dTextPipe;
    protected static D3DDrawImage d3dImagePipe;
    
    private native boolean initTexture(long p0, boolean p1, boolean p2);
    
    private native boolean initFlipBackbuffer(long p0, long p1, int p2, int p3, int p4);
    
    private native boolean initRTSurface(long p0, boolean p1);
    
    private native void initOps(int p0, int p1, int p2);
    
    protected D3DSurfaceData(WComponentPeer peer, D3DGraphicsConfig d3DGraphicsConfig, int width, int height, Image offscreenImage, ColorModel cm, int backBuffersNum, int swapEffect, ExtendedBufferCapabilities.VSyncType syncType, int type) {
        super(getCustomSurfaceType(type), cm);
        this.graphicsDevice = d3DGraphicsConfig.getD3DDevice();
        this.peer = peer;
        this.type = type;
        this.width = width;
        this.height = height;
        this.offscreenImage = offscreenImage;
        this.backBuffersNum = backBuffersNum;
        this.swapEffect = swapEffect;
        this.syncType = syncType;
        this.initOps(this.graphicsDevice.getScreen(), width, height);
        if (type == 1) {
            this.setSurfaceLost(true);
        }
        else {
            this.initSurface();
        }
        this.setBlitProxyKey(d3DGraphicsConfig.getProxyKey());
    }
    
    @Override
    public SurfaceDataProxy makeProxyFor(SurfaceData surfaceData) {
        return D3DSurfaceDataProxy.createProxy(surfaceData, (D3DGraphicsConfig)this.graphicsDevice.getDefaultConfiguration());
    }
    
    public static D3DSurfaceData createData(WComponentPeer wComponentPeer, Image image) {
        D3DGraphicsConfig gc = getGC(wComponentPeer);
        if (gc == null || !wComponentPeer.isAccelCapable()) {
            return null;
        }
        BufferCapabilities backBufferCaps = wComponentPeer.getBackBufferCaps();
        ExtendedBufferCapabilities.VSyncType vSyncType = ExtendedBufferCapabilities.VSyncType.VSYNC_DEFAULT;
        if ((backBufferCaps instanceof ExtendedBufferCapabilities)) {
            vSyncType = ((ExtendedBufferCapabilities)backBufferCaps).getVSync();
        }
        Rectangle bounds = wComponentPeer.getBounds();
        BufferCapabilities.FlipContents flipContents = backBufferCaps.getFlipContents();
        int n;
        if (flipContents == BufferCapabilities.FlipContents.COPIED) {
            n = 3;
        }
        else if (flipContents == BufferCapabilities.FlipContents.PRIOR) {
            n = 2;
        }
        else {
            n = 1;
        }
        return new D3DSurfaceData(wComponentPeer, gc, bounds.width, bounds.height, image, wComponentPeer.getColorModel(), wComponentPeer.getBackBuffersNum(), n, vSyncType, 4);
    }
    
    public static D3DSurfaceData createData(WComponentPeer wComponentPeer) {
        D3DGraphicsConfig gc = getGC(wComponentPeer);
        if (gc == null || !wComponentPeer.isAccelCapable()) {
            return null;
        }
        return new D3DSurfaceData.D3DWindowSurfaceData(wComponentPeer, gc);
    }
    
    public static D3DSurfaceData createData(D3DGraphicsConfig d3DGraphicsConfig, int n, int n2, ColorModel colorModel, Image image, int n3) {
        if (n3 == 5 && !d3DGraphicsConfig.getD3DDevice().isCapPresent((colorModel.getTransparency() == 1) ? 8 : 4)) {
            n3 = 2;
        }
        Object o = null;
        try {
            o = new D3DSurfaceData(null, d3DGraphicsConfig, n, n2, image, colorModel, 0, 1, ExtendedBufferCapabilities.VSyncType.VSYNC_DEFAULT, n3);
        }
        catch (InvalidPipeException ex) {
            if (n3 == 5 && ((SunVolatileImage)image).getForcedAccelSurfaceType() != 5) {
                n3 = 2;
                o = new D3DSurfaceData(null, d3DGraphicsConfig, n, n2, image, colorModel, 0, 1, ExtendedBufferCapabilities.VSyncType.VSYNC_DEFAULT, n3);
            }
        }
        return o;
    }
    
    private static SurfaceType getCustomSurfaceType(int n) {
        switch (n) {
            case 3: {
                return D3DSurfaceData.D3DTexture;
            }
            case 5: {
                return D3DSurfaceData.D3DSurfaceRTT;
            }
            default: {
                return D3DSurfaceData.D3DSurface;
            }
        }
    }
    
    private boolean initSurfaceNow() {
        int n = this.getTransparency() == 1;
        switch (this.type) {
            case 2: {
                return this.initRTSurface(this.getNativeOps(), (boolean)n);
            }
            case 3: {
                return this.initTexture(this.getNativeOps(), false, (boolean)n);
            }
            case 5: {
                return this.initTexture(this.getNativeOps(), true, (boolean)n);
            }
            case 1:
            case 4: {
                return this.initFlipBackbuffer(this.getNativeOps(), this.peer.getData(), this.backBuffersNum, this.swapEffect, this.syncType.id());
            }
            default: {
                return false;
            }
        }
    }
    
    protected void initSurface() {
        synchronized (this) {
            this.wrn = null;
        }
        class Status
        {
            boolean success;
            
            Status() {
                this.success = false;
            }
        }
        final D3DSurfaceData.1Status status = new Status();
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            instance.flushAndInvokeNow(new Runnable() {
                @Override
                public void run() {
                    status.success = D3DSurfaceData.access$000(D3DSurfaceData.this);
                }
            });
            if (!status.success) {
                throw new InvalidPipeException("Error creating D3DSurface");
            }
        }
        finally {
            instance.unlock();
        }
    }
    
    @Override
    public final D3DContext getContext() {
        return this.graphicsDevice.getContext();
    }
    
    @Override
    public final int getType() {
        return this.type;
    }
    
    private static native int dbGetPixelNative(long p0, int p1, int p2);
    
    private static native void dbSetPixelNative(long p0, int p1, int p2, int p3);
    
    @Override
    public synchronized Raster getRaster(int n, int n2, int n3, int n4) {
        if (this.wrn == null) {
            DirectColorModel directColorModel = (DirectColorModel)this.getColorModel();
            int width = this.width;
            int dataType;
            if (directColorModel.getPixelSize() > 16) {
                dataType = 3;
            }
            else {
                dataType = 1;
            }
            this.wrn = WritableRasterNative.createNativeRaster(new SinglePixelPackedSampleModel(dataType, this.width, this.height, width, directColorModel.getMasks()), new D3DSurfaceData.D3DDataBufferNative(this, dataType, this.width, this.height));
        }
        return this.wrn;
    }
    
    @Override
    public boolean canRenderLCDText(SunGraphics2D sunGraphics2D) {
        return this.graphicsDevice.isCapPresent(65536) && sunGraphics2D.compositeState <= 0 && sunGraphics2D.paintState <= 0 && sunGraphics2D.surfaceData.getTransparency() == 1;
    }
    
    void disableAccelerationForSurface() {
        if (this.offscreenImage != null) {
            SurfaceManager manager = SurfaceManager.getManager(this.offscreenImage);
            if ((manager instanceof D3DVolatileSurfaceManager)) {
                this.setSurfaceLost(true);
                ((D3DVolatileSurfaceManager)manager).setAccelerationEnabled(false);
            }
        }
    }
    
    @Override
    public void validatePipe(SunGraphics2D sunGraphics2D) {
        int n = 0;
        if (sunGraphics2D.compositeState >= 2) {
            super.validatePipe(sunGraphics2D);
            sunGraphics2D.imagepipe = D3DSurfaceData.d3dImagePipe;
            this.disableAccelerationForSurface();
            return;
        }
        D3DTextRenderer textpipe;
        if ((sunGraphics2D.compositeState <= 0 && sunGraphics2D.paintState <= 1) || (sunGraphics2D.compositeState == 1 && sunGraphics2D.paintState <= 1 && ((AlphaComposite)sunGraphics2D.composite).getRule() == 3) || (sunGraphics2D.compositeState == 2 && sunGraphics2D.paintState <= 1)) {
            textpipe = D3DSurfaceData.d3dTextPipe;
        }
        else {
            super.validatePipe(sunGraphics2D);
            textpipe = sunGraphics2D.textpipe;
            n = 1;
        }
        PixelToParallelogramConverter pixelToParallelogramConverter = null;
        D3DRenderer fillpipe = null;
        if (sunGraphics2D.antialiasHint != 2) {
            if (sunGraphics2D.paintState <= 1) {
                if (sunGraphics2D.compositeState <= 2) {
                    pixelToParallelogramConverter = D3DSurfaceData.d3dTxRenderPipe;
                    fillpipe = D3DSurfaceData.d3dRenderPipe;
                }
            }
            else if (sunGraphics2D.compositeState <= 1 && D3DPaints.isValid(sunGraphics2D)) {
                pixelToParallelogramConverter = D3DSurfaceData.d3dTxRenderPipe;
                fillpipe = D3DSurfaceData.d3dRenderPipe;
            }
        }
        else if (sunGraphics2D.paintState <= 1) {
            if (this.graphicsDevice.isCapPresent(524288) && (sunGraphics2D.imageComp == CompositeType.SrcOverNoEa || sunGraphics2D.imageComp == CompositeType.SrcOver)) {
                if (n == 0) {
                    super.validatePipe(sunGraphics2D);
                    n = 1;
                }
                PixelToParallelogramConverter shapepipe = new PixelToParallelogramConverter(sunGraphics2D.shapepipe, D3DSurfaceData.d3dAAPgramPipe, 0.125, 0.499, false);
                sunGraphics2D.drawpipe = shapepipe;
                sunGraphics2D.fillpipe = shapepipe;
                sunGraphics2D.shapepipe = shapepipe;
            }
            else if (sunGraphics2D.compositeState == 2) {
                pixelToParallelogramConverter = D3DSurfaceData.d3dTxRenderPipe;
                fillpipe = D3DSurfaceData.d3dRenderPipe;
            }
        }
        if (pixelToParallelogramConverter != null) {
            if (sunGraphics2D.transformState >= 3) {
                sunGraphics2D.drawpipe = pixelToParallelogramConverter;
                sunGraphics2D.fillpipe = pixelToParallelogramConverter;
            }
            else if (sunGraphics2D.strokeState != 0) {
                sunGraphics2D.drawpipe = pixelToParallelogramConverter;
                sunGraphics2D.fillpipe = fillpipe;
            }
            else {
                sunGraphics2D.drawpipe = fillpipe;
                sunGraphics2D.fillpipe = fillpipe;
            }
            sunGraphics2D.shapepipe = pixelToParallelogramConverter;
        }
        else if (n == 0) {
            super.validatePipe(sunGraphics2D);
        }
        sunGraphics2D.textpipe = textpipe;
        sunGraphics2D.imagepipe = D3DSurfaceData.d3dImagePipe;
        return;
    }
    
    @Override
    protected MaskFill getMaskFill(SunGraphics2D sg2d) {
        if (sg2d.paintState > 1 && (!D3DPaints.isValid(sg2d) || !this.graphicsDevice.isCapPresent(16))) {
            return null;
        }
        return super.getMaskFill(sg2d);
    }
    
    @Override
    public boolean copyArea(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        if (sunGraphics2D.transformState < 3 && sunGraphics2D.compositeState < 2) {
            n += sunGraphics2D.transX;
            n2 += sunGraphics2D.transY;
            D3DSurfaceData.d3dRenderPipe.copyArea(sunGraphics2D, n, n2, n3, n4, n5, n6);
            return true;
        }
        return false;
    }
    
    @Override
    public void flush() {
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            RenderBuffer buffer = instance.getBuffer();
            instance.ensureCapacityAndAlignment(12, 4);
            buffer.putInt(72);
            buffer.putLong(this.getNativeOps());
            instance.flushNow();
        }
        finally {
            instance.unlock();
        }
    }
    
    static void dispose(long x) {
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            RenderBuffer buffer = instance.getBuffer();
            instance.ensureCapacityAndAlignment(12, 4);
            buffer.putInt(73);
            buffer.putLong(x);
            instance.flushNow();
        }
        finally {
            instance.unlock();
        }
    }
    
    static void swapBuffers(D3DSurfaceData d3DSurfaceData, final int x, final int x2, final int x3, final int x4) {
        long nativeOps = d3DSurfaceData.getNativeOps();
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        if (D3DRenderQueue.isRenderQueueThread()) {
            if (!instance.tryLock()) {
                final Component target = (Component)d3DSurfaceData.getPeer().getTarget();
                SunToolkit.executeOnEventHandlerThread(target, new Runnable() {
                    @Override
                    public void run() {
                        target.repaint(x, x2, x3, x4);
                    }
                });
                return;
            }
        }
        else {
            instance.lock();
        }
        try {
            RenderBuffer buffer = instance.getBuffer();
            instance.ensureCapacityAndAlignment(28, 4);
            buffer.putInt(80);
            buffer.putLong(nativeOps);
            buffer.putInt(x);
            buffer.putInt(x2);
            buffer.putInt(x3);
            buffer.putInt(x4);
            instance.flushNow();
        }
        finally {
            instance.unlock();
        }
    }
    
    @Override
    public Object getDestination() {
        return this.offscreenImage;
    }
    
    @Override
    public Rectangle getBounds() {
        if (this.type == 4 || this.type == 1) {
            Rectangle bounds;
            Rectangle rectangle = bounds = this.peer.getBounds();
            int n = 0;
            rectangle.y = n;
            bounds.x = n;
            return rectangle;
        }
        return new Rectangle(this.width, this.height);
    }
    
    @Override
    public Rectangle getNativeBounds() {
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            return new Rectangle(this.nativeWidth, this.nativeHeight);
        }
        finally {
            instance.unlock();
        }
    }
    
    @Override
    public GraphicsConfiguration getDeviceConfiguration() {
        return this.graphicsDevice.getDefaultConfiguration();
    }
    
    @Override
    public SurfaceData getReplacement() {
        return SurfaceData.restoreContents(this.offscreenImage);
    }
    
    private static D3DGraphicsConfig getGC(WComponentPeer wComponentPeer) {
        if (wComponentPeer != null) {
            GraphicsConfiguration graphicsConfiguration = wComponentPeer.getGraphicsConfiguration();
        }
        else {
            GraphicsConfiguration graphicsConfiguration = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
        }
        GraphicsConfiguration graphicsConfiguration;
        return (graphicsConfiguration instanceof D3DGraphicsConfig) ? ((D3DGraphicsConfig)graphicsConfiguration) : null;
    }
    
    void restoreSurface() {
        this.initSurface();
    }
    
    WComponentPeer getPeer() {
        return this.peer;
    }
    
    @Override
    public void setSurfaceLost(boolean surfaceLost) {
        super.setSurfaceLost(surfaceLost);
        if (surfaceLost && this.offscreenImage != null) {
            SurfaceManager.getManager(this.offscreenImage).acceleratedSurfaceLost();
        }
    }
    
    private static native long getNativeResourceNative(long p0, int p1);
    
    @Override
    public long getNativeResource(int n) {
        return getNativeResourceNative(this.getNativeOps(), n);
    }
    
    public static native boolean updateWindowAccelImpl(long p0, long p1, int p2, int p3);
    
    static boolean access$000(D3DSurfaceData d3DSurfaceData) {
        return d3DSurfaceData.initSurfaceNow();
    }
    
    static int access$100(long n, int n2, int n3) {
        return dbGetPixelNative(n, n2, n3);
    }
    
    static void access$200(long n, int n2, int n3, int n4) {
        dbSetPixelNative(n, n2, n3, n4);
    }
    
    static {
        D3DSurface = SurfaceType.Any.deriveSubType("D3D Surface", PixelConverter.ArgbPre.instance);
        D3DSurfaceRTT = D3DSurfaceData.D3DSurface.deriveSubType("D3D Surface (render-to-texture)");
        D3DTexture = SurfaceType.Any.deriveSubType("D3D Texture");
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        D3DSurfaceData.d3dImagePipe = new D3DDrawImage();
        D3DSurfaceData.d3dTextPipe = new D3DTextRenderer(instance);
        D3DSurfaceData.d3dRenderPipe = new D3DRenderer(instance);
        if (GraphicsPrimitive.tracingEnabled()) {
            D3DSurfaceData.d3dTextPipe = D3DSurfaceData.d3dTextPipe.traceWrap();
            D3DSurfaceData.d3dRenderPipe = D3DSurfaceData.d3dRenderPipe.traceWrap();
        }
        D3DSurfaceData.d3dAAPgramPipe = D3DSurfaceData.d3dRenderPipe.getAAParallelogramPipe();
        D3DSurfaceData.d3dTxRenderPipe = new PixelToParallelogramConverter(D3DSurfaceData.d3dRenderPipe, D3DSurfaceData.d3dRenderPipe, 1.0, 0.25, true);
        D3DBlitLoops.register();
        D3DMaskFill.register();
        D3DMaskBlit.register();
    }
    
    static class D3DDataBufferNative extends DataBufferNative
    {
        int pixel;
        
        protected D3DDataBufferNative(SurfaceData sData, int type, int width, int height) {
            super(sData, type, width, height);
        }
        
        @Override
        protected int getElem(int n, int n2, SurfaceData surfaceData) {
            if (surfaceData.isSurfaceLost()) {
                return 0;
            }
            D3DRenderQueue instance = D3DRenderQueue.getInstance();
            instance.lock();
            int pixel2;
            try {
                instance.flushAndInvokeNow(new Runnable() {
                    @Override
                    public void run() {
                        D3DSurfaceData.D3DDataBufferNative.this.pixel = D3DSurfaceData.access$100(surfaceData.getNativeOps(), n, n2);
                    }
                });
                int pixel = this.pixel;
            }
            finally {
                pixel2 = this.pixel;
                instance.unlock();
            }
            return pixel2;
        }
        
        @Override
        protected void setElem(int n, int n2, int n3, SurfaceData surfaceData) {
            if (surfaceData.isSurfaceLost()) {
                return;
            }
            D3DRenderQueue instance = D3DRenderQueue.getInstance();
            instance.lock();
            try {
                instance.flushAndInvokeNow(new Runnable() {
                    @Override
                    public void run() {
                        D3DSurfaceData.access$200(surfaceData.getNativeOps(), n, n2, n3);
                    }
                });
                surfaceData.markDirty();
            }
            finally {
                instance.unlock();
            }
            return;
        }
    }
    
    public static class D3DWindowSurfaceData extends D3DSurfaceData
    {
        StateTracker dirtyTracker;
        
        public D3DWindowSurfaceData(WComponentPeer wComponentPeer, D3DGraphicsConfig d3DGraphicsConfig) {
            super(wComponentPeer, d3DGraphicsConfig, wComponentPeer.getBounds().width, wComponentPeer.getBounds().height, null, wComponentPeer.getColorModel(), 1, 3, ExtendedBufferCapabilities.VSyncType.VSYNC_DEFAULT, 1);
            this.dirtyTracker = this.getStateTracker();
        }
        
        @Override
        public SurfaceData getReplacement() {
            return ScreenUpdateManager.getInstance().getReplacementScreenSurface(this.peer, this);
        }
        
        @Override
        public Object getDestination() {
            return this.peer.getTarget();
        }
        
        @Override
        void disableAccelerationForSurface() {
            this.setSurfaceLost(true);
            this.invalidate();
            this.flush();
            this.peer.disableAcceleration();
            ScreenUpdateManager.getInstance().dropScreenSurface(this);
        }
        
        @Override
        void restoreSurface() {
            if (!this.peer.isAccelCapable()) {
                throw new InvalidPipeException("Onscreen acceleration disabled for this surface");
            }
            Window fullScreenWindow = this.graphicsDevice.getFullScreenWindow();
            if (fullScreenWindow != null && fullScreenWindow != this.peer.getTarget()) {
                throw new InvalidPipeException("Can't restore onscreen surface when in full-screen mode");
            }
            super.restoreSurface();
            this.setSurfaceLost(false);
            D3DRenderQueue instance = D3DRenderQueue.getInstance();
            instance.lock();
            try {
                super.getContext().invalidateContext();
            }
            finally {
                instance.unlock();
            }
            return;
        }
        
        public boolean isDirty() {
            return !this.dirtyTracker.isCurrent();
        }
        
        public void markClean() {
            this.dirtyTracker = this.getStateTracker();
        }
    }
}
